<?php
/**
 * Created by PhpStorm.
 * User: benno
 * Date: 2/14/14
 * Time: 10:29 AM
 */

class Rest
{
    /**
     * @var string username
     */
    protected $username;

    /**
     * @var string password
     */
    protected $password;

    /**
     * @var int database
     */
    protected $database;

    /**
     * @var string token
     */
    protected $token;

    /**
     * @var Curl
     */
    protected $curl;

    const API_ENDPOINT = 'https://mail.e-mark.nl/rest/v2/';

    const API_BULK_ENDPOINT = 'https://mail.e-mark.nl/rest/api/bulk/v2/call';
    //const API_ENDPOINT = 'http://blatrunk.local/rest/v2/';

    /**
     * @var bool debugging
     */
    private $debug = false;

    /**
     * @param $username
     * @param $password
     * @param $database
     */
    public function __construct($username, $password, $database)
    {
        $this->setUsername($username);
        $this->setPassword($password);
        $this->setDatabase($database);
        $this->createToken();
    }

    /**
     * Create Token to log in on the API
     * @throws Exception
     */
    protected function createToken()
    {
        $url = Rest::API_ENDPOINT . 'user/login';
        $data = array(
            'username'  => $this->getUsername(),
            'password'  => $this->getPassword()
        );

        $result = $this->parseData($this->runCurl($url, $data));

        if(!$result->valid){
            throw new Exception('Can\'t log in');
        }

        $this->setToken($result->data->token);
    }


    public function callBulk($actions)
    {
        $bulkData = array(
            'credentials' => array(
                'username' => $this->getUsername(),
                'password' => $this->getPassword()
            ),
            'database' => $this->getDatabase(),
            'halt_on_error' => false,
            'skip_result' => false,
            'actions' => $actions
        );

        $url = Rest::API_BULK_ENDPOINT;

        $result = $this->parseData($this->runCurl($url, $bulkData));

    }



    public function call($action, $data = array(), $httpVerb = 'GET' )
    {
        $url = Rest::API_ENDPOINT . $action;
        $result = $this->parseData($this->runCurl($url, $data));
        return $result;
    }


    /**
     * @param string $json
     * @return stdClass
     */
    protected function parseData($json)
    {
//        var_dump($json);
        return json_decode($json);
    }


    /**
     * Do the actual curl request
     * @param $url
     * @param $data
     * @param string $verb
     * @return mixed
     */
    protected function runCurl($url, $data, $verb = 'GET')
    {
        $ch = $this->initCurl();

        if(!is_null($this->getToken())){
            $data['token'] = $this->getToken();
            $data['database'] = $this->getDatabase();
        }

        if($verb == 'GET'){
            $url .= '?' . http_build_query($data);
        }

        echo PHP_EOL;
        echo urldecode($url);
        echo PHP_EOL;

        curl_setopt($this->getCurl(), CURLOPT_URL, $url);
        $data = curl_exec($this->getCurl());




        $httpResponse = curl_getinfo($this->getCurl(), CURLINFO_HTTP_CODE);

        if($this->debug === true){
            echo $httpResponse . ': ' . $url . PHP_EOL;
        }

        return $data;
    }


    /**
     * @return Curl
     */
    protected function initCurl()
    {
        if(is_null($this->getCurl())){
            $ch = curl_init();
            curl_setopt_array($ch, array(
                CURLOPT_HEADER => false,
                CURLOPT_RETURNTRANSFER => true,
                CURLOPT_USERAGENT => 'Emark Mail',
                CURLOPT_SSL_VERIFYPEER => false,
                CURLOPT_HTTPHEADER => array("Content-Type: application/x-www-form-urlencoded"),
                CURLINFO_HEADER_OUT => false,
                CURLOPT_FAILONERROR => false,
            ));

         //   if($this->debug === true){
                curl_setopt($ch, CURLOPT_VERBOSE, true);
        //    }
            $this->setCurl($ch);
        }

        return $this->getCurl();
    }


    /**
     * @param mixed $password
     */
    public function setPassword($password)
    {
        $this->password = $password;
    }

    /**
     * @return mixed
     */
    public function getPassword()
    {
        return $this->password;
    }

    /**
     * @param mixed $username
     */
    public function setUsername($username)
    {
        $this->username = $username;
    }

    /**
     * @return mixed
     */
    public function getUsername()
    {
        return $this->username;
    }

    /**
     * @param mixed $database
     */
    public function setDatabase($database)
    {
        $this->database = $database;
    }

    /**
     * @return mixed
     */
    public function getDatabase()
    {
        return $this->database;
    }

    /**
     * @param mixed $curl
     */
    public function setCurl($curl)
    {
        $this->curl = $curl;
    }

    /**
     * @return mixed
     */
    public function getCurl()
    {
        return $this->curl;
    }

    /**
     * @param mixed $token
     */
    public function setToken($token)
    {
        $this->token = $token;
    }

    /**
     * @return mixed
     */
    public function getToken()
    {
        return $this->token;
    }
} 
